@@ -42,6 +42,8 @@
 extern "C" {
 #endif
 
+#include <alsa/asoundlib.h>
+
 /**
  *  \defgroup ucm Use Case Interface
  *  The ALSA Use Case manager interface.
@@ -84,7 +86,7 @@ extern "C" {
  *  + Get the TQ parameter for each use case verb, use case device and
  *     modifier.
  *  + Get the ALSA master playback and capture volume/switch kcontrols
- *     for each use case.
+ *     or mixer elements for each use case.
  */
 
 
@@ -112,10 +114,25 @@ extern "C" {
  *
  * Physical system devices the render and capture audio. Devices can be OR'ed
  * together to support audio on simultaneous devices.
+ *
+ * If multiple devices with the same name exists, the number suffixes should
+ * be added to these names like HDMI1,HDMI2,HDMI3 etc. No number gaps are
+ * allowed. The names with numbers must be continuous. It is allowed to put
+ * a whitespace between name and index (like 'Line 1') for the better
+ * readability. The device names 'Line 1' and 'Line1' are equal for
+ * this purpose.
+ *
+ * If EnableSequence/DisableSequence controls independent paths in the hardware
+ * it is also recommended to split playback and capture UCM devices and use
+ * the number suffixes. Example use case: Use the integrated microphone
+ * in the laptop instead the microphone in headphones.
+ *
+ * The preference of the devices is determined by the priority value.
  */
 #define SND_USE_CASE_DEV_NONE		"None"		/**< None Device */
 #define SND_USE_CASE_DEV_SPEAKER	"Speaker"	/**< Speaker Device */
 #define SND_USE_CASE_DEV_LINE		"Line"		/**< Line Device */
+#define SND_USE_CASE_DEV_MIC            "Mic"           /**< Microphone Device */
 #define SND_USE_CASE_DEV_HEADPHONES	"Headphones"	/**< Headphones Device */
 #define SND_USE_CASE_DEV_HEADSET	"Headset"	/**< Headset Device */
 #define SND_USE_CASE_DEV_HANDSET	"Handset"	/**< Handset Device */
@@ -123,6 +140,7 @@ extern "C" {
 #define SND_USE_CASE_DEV_EARPIECE	"Earpiece"	/**< Earpiece Device */
 #define SND_USE_CASE_DEV_SPDIF		"SPDIF"		/**< SPDIF Device */
 #define SND_USE_CASE_DEV_HDMI		"HDMI"		/**< HDMI Device */
+#define SND_USE_CASE_DEV_USB		"USB"		/**< USB Device (multifunctional) */
 /* add new devices to end of list */
 
 
@@ -132,6 +150,13 @@ extern "C" {
  * The use case modifier allows runtime configuration changes to deal with
  * asynchronous events.
  *
+ * If multiple modifiers with the same name exists, the number suffixes should
+ * be added to these names like 'Echo Reference 1','Echo Reference 2' etc.
+ * No number gaps are allowed. The names with numbers must be continuous.
+ * It is allowed to put a whitespace between name and index for the better
+ * readability. The modifier names 'Something 1' and 'Something1' are equal
+ * for this purpose.
+ *
  * e.g. to record a voice call :-
  *  1. Set verb to SND_USE_CASE_VERB_VOICECALL (for voice call)
  *  2. Set modifier SND_USE_CASE_MOD_CAPTURE_VOICE when capture required.
@@ -204,6 +229,7 @@ int snd_use_case_free_list(const char *list[], int items);
  *   - _enadevs			- get list of enabled devices
  *   - _enamods			- get list of enabled modifiers
  *
+ *   - _identifiers/{modifier}|{device}[/{verb}]     - list of value identifiers
  *   - _supporteddevs/{modifier}|{device}[/{verb}]   - list of supported devices
  *   - _conflictingdevs/{modifier}|{device}[/{verb}] - list of conflicting devices
  *
@@ -257,8 +283,17 @@ int snd_use_case_get_list(snd_use_case_mgr_t *uc_mgr,
  *                              "=Variable/Modifier/Verb"
  *
  * Recommended names for values:
+ *   - Linked
+ *      - value "True" or "1" (case insensitive)
+ *      - this is a linked UCM card
+ *      - don't use this UCM card, because the other UCM card refers devices
+ *      - valid only in the ValueDefaults section (query '=Linked')
  *   - TQ
  *      - Tone Quality
+ *   - Priority
+ *      - priority value (1-10000), higher value means higher priority
+ *      - valid only for verbs
+ *      - for devices - PlaybackPriority and CapturePriority
  *   - PlaybackPCM
  *      - full PCM playback device name
  *   - PlaybackPCMIsDummy
@@ -280,9 +315,13 @@ int snd_use_case_get_list(snd_use_case_mgr_t *uc_mgr,
  *   - PlaybackCTL
  *      - playback control device name
  *   - PlaybackVolume
- *      - playback control volume ID string
+ *      - playback control volume identifier string
+ *	- can be parsed using snd_use_case_parse_ctl_elem_id()
  *   - PlaybackSwitch
- *      - playback control switch ID string
+ *      - playback control switch identifier string
+ *	- can be parsed using snd_use_case_parse_ctl_elem_id()
+ *   - PlaybackPriority
+ *      - priority value (1-10000), higher value means higher priority
  *   - CaptureRate
  *      - capture device sample rate
  *   - CaptureChannels
@@ -290,36 +329,67 @@ int snd_use_case_get_list(snd_use_case_mgr_t *uc_mgr,
  *   - CaptureCTL
  *      - capture control device name
  *   - CaptureVolume
- *      - capture control volume ID string
+ *      - capture control volume identifier string
+ *	- can be parsed using snd_use_case_parse_ctl_elem_id()
  *   - CaptureSwitch
- *      - capture control switch ID string
+ *      - capture control switch identifier string
+ *	- can be parsed using snd_use_case_parse_ctl_elem_id()
+ *   - CapturePriority
+ *      - priority value (1-10000), higher value means higher priority
  *   - PlaybackMixer
  *      - name of playback mixer
- *   - PlaybackMixerID
- *      - mixer playback ID
+ *   - PlaybackMixerElem
+ *      - mixer element playback identifier
+ *	- can be parsed using snd_use_case_parse_selem_id()
+ *   - PlaybackMasterElem
+ *      - mixer element playback identifier for the master control
+ *	- can be parsed using snd_use_case_parse_selem_id()
+ *   - PlaybackMasterType
+ *      - type of the master volume control
+ *      - Valid values: "soft" (software attenuation)
  *   - CaptureMixer
  *      - name of capture mixer
- *   - CaptureMixerID
- *      - mixer capture ID
- *   - JackControl, JackDev, JackHWMute
- *      - Jack information for a device. The jack status can be reported via
- *        a kcontrol and/or via an input device. **JackControl** is the
- *        kcontrol name of the jack, and **JackDev** is the input device id of
- *        the jack (if the full input device path is /dev/input/by-id/foo, the
- *        JackDev value should be "foo"). UCM configuration files should
- *        contain both JackControl and JackDev when possible, because
- *        applications are likely to support only one or the other.
- *
- *        If **JackHWMute** is set, it indicates that when the jack is plugged
- *        in, the hardware automatically mutes some other device(s). The
- *        JackHWMute value is a space-separated list of device names (this
- *        isn't compatible with device names with spaces in them, so don't use
- *        such device names!). Note that JackHWMute should be used only when
- *        the hardware enforces the automatic muting. If the hardware doesn't
- *        enforce any muting, it may still be tempting to set JackHWMute to
- *        trick upper software layers to e.g. automatically mute speakers when
- *        headphones are plugged in, but that's application policy
- *        configuration that doesn't belong to UCM configuration files.
+ *   - CaptureMixerElem
+ *      - mixer element capture identifier
+ *	- can be parsed using snd_use_case_parse_selem_id()
+ *   - CaptureMasterElem
+ *      - mixer element playback identifier for the master control
+ *	- can be parsed using snd_use_case_parse_selem_id()
+ *   - CaptureMasterType
+ *      - type of the master volume control
+ *      - Valid values: "soft" (software attenuation)
+ *   - EDIDFile
+ *      - Path to EDID file for HDMI devices
+ *   - JackCTL
+ *      - jack control device name
+ *   - JackControl
+ *      - jack control identificator
+ *      - can be parsed using snd_use_case_parse_ctl_elem_id()
+ *      - UCM configuration files should contain both JackControl and JackDev
+ *        when possible, because applications are likely to support only one
+ *        or the other
+ *   - JackDev
+ *      - the input device id of the jack (if the full input device path is
+ *        /dev/input/by-id/foo, the JackDev value should be "foo")
+ *      - UCM configuration files should contain both JackControl and JackDev
+ *        when possible, because applications are likely to support only one
+ *        or the other
+ *   - JackHWMute
+ *	  If this value is set, it indicates that when the jack is plugged
+ *	  in, the hardware automatically mutes some other device(s). The
+ *	  value is a space-separated list of device names. If the device
+ *	  name contains space, it must be enclosed to ' or ", e.g.:
+ *		JackHWMute "'Dock Headphone' Headphone"
+ *        Note that JackHWMute should be used only when the hardware enforces
+ *        the automatic muting. If the hardware doesn't enforce any muting, it
+ *        may still be tempting to set JackHWMute to trick upper software layers
+ *        to e.g. automatically mute speakers when headphones are plugged in,
+ *        but that's application policy configuration that doesn't belong
+ *        to UCM configuration files.
+ *   - MinBufferLevel
+ *	- This is used on platform where reported buffer level is not accurate.
+ *	  E.g. "512", which holds 512 samples in device buffer. Note: this will
+ *	  increase latency.
  */
 int snd_use_case_get(snd_use_case_mgr_t *uc_mgr,
                      const char *identifier,
@@ -348,6 +418,8 @@ int snd_use_case_geti(snd_use_case_mgr_t *uc_mgr,
  * \return Zero if success, otherwise a negative error code
  *
  * Known identifiers:
+ *   - _boot			- execute the boot sequence (value = NULL)
+ *   - _defaults		- execute the 'defaults' sequence (value = NULL)
  *   - _verb			- set current verb = value
  *   - _enadev			- enable given device = value
  *   - _disdev			- disable given device = value
@@ -371,8 +443,25 @@ int snd_use_case_set(snd_use_case_mgr_t *uc_mgr,
  * \param uc_mgr Returned use case manager pointer
  * \param card_name Sound card name.
  * \return zero if success, otherwise a negative error code
+ *
+ * By default only first card is used when the driver card
+ * name or long name is passed in the card_name argument.
+ *
+ * The "strict:" prefix in the card_name defines that
+ * there is no driver name / long name matching. The straight
+ * configuration is used.
+ *
+ * The "hw:" prefix in the card_name will load the configuration
+ * for the ALSA card specified by the card index (value) or
+ * the card string identificator.
+ *
+ * The sound card might be also composed from several physical
+ * sound cards (for the default and strict card_name).
+ * The application cannot expect that the device names will refer
+ * only one ALSA sound card in this case.
  */
-int snd_use_case_mgr_open(snd_use_case_mgr_t **uc_mgr, const char *card_name);
+int snd_use_case_mgr_open(snd_use_case_mgr_t **uc_mgr,
+                          const char *card_name);
 
 
 /**
@@ -422,6 +511,28 @@ static __inline__ int snd_use_case_verb_list(snd_use_case_mgr_t *uc_mgr,
 	return snd_use_case_get_list(uc_mgr, "_verbs", list);
 }
 
+/**
+ * \brief Parse control element identifier
+ * \param elem_id Element identifier
+ * \param ucm_id Use case identifier
+ * \param value String value to be parsed
+ * \return Zero if success, otherwise a negative error code
+ */
+int snd_use_case_parse_ctl_elem_id(snd_ctl_elem_id_t *dst,
+				   const char *ucm_id,
+				   const char *value);
+
+/**
+ * \brief Parse mixer element identifier
+ * \param dst Simple mixer element identifier
+ * \param ucm_id Use case identifier
+ * \param value String value to be parsed
+ * \return Zero if success, otherwise a negative error code
+ */
+int snd_use_case_parse_selem_id(snd_mixer_selem_id_t *dst,
+				const char *ucm_id,
+				const char *value);
+
 /**
  *  \}
  */
